import React, { Component } from 'react';

import InfoPanel from './InfoPanel.jsx';
import ControlPanel from './ControlPanel.jsx';
import Tooltip from './Tooltip.jsx';
import OgmaContainer from './OgmaComponent.jsx';
import initialGraph from './data';

// import './App.css';

// external helpers
function createNode(id) {
  return {
    id,
    attributes: {
      color: id % 2 ? 'purple' : 'orange'
    }
  };
}

function createEdge(node, nodesInGraph) {
  // pick a random node in the graph
  const randomIndex = Math.floor(Math.random() * nodesInGraph.length);
  const otherNode = nodesInGraph[randomIndex];
  return {
    id: `${otherNode.id}-${node.id}`,
    source: otherNode.id,
    target: node.id
  };
}

export default class App extends Component {
  state = {
    counter: initialGraph.nodes.length,
    graph: initialGraph,
    // which layout to use?
    layout: 'force',
    // tooltip state
    tooltipPosition: null,
    tooltipNodeId: null
  };

  addNode = () => {
    const newNode = createNode(this.state.counter);
    // note: we're creating a new array here, adding an extra node at the end
    const nodes = [...this.state.graph.nodes, newNode];
    const edges = [
      ...this.state.graph.edges,
      createEdge(newNode, this.state.graph.nodes)
    ];
    // use the counter as id
    this.setState({
      graph: { nodes, edges },
      counter: this.state.counter + 1
    });
  };

  onLayoutChange = ({ target }) =>
    this.setState({
      layout: target.value
    });

  onNodeUnHover = () => {
    // clear the tooltip state
    return this.setState({
      tooltipPosition: null,
      tooltipNodeId: null
    });
  };

  onNodeHover = ({ x, y, target }) => {
    if (target.isNode) {
      // save the tooltip state (offset by 20px the vertical position)
      this.setState({
        tooltipPosition: { x, y: y + 20 },
        tooltipNodeId: target.getId()
      });
    }
  };

  handleTooltip() {
    // check if there's a position stored for the tooltip
    const nodeTooltip = this.state.tooltipPosition
      ? this.state.graph.nodes.find(
          node => node.id === this.state.tooltipNodeId
        )
      : null;

    if (!nodeTooltip) {
      // nothing in the state, drop it
      return;
    }
    // show the tooltip
    return <Tooltip position={this.state.tooltipPosition} data={nodeTooltip} />;
  }

  render() {
    return (
      <div className="App">
        <OgmaContainer
          data={this.state.graph}
          onHover={this.onNodeHover}
          onUnhover={this.onNodeUnHover}
          layout={this.state.layout}
        />
        {this.handleTooltip()}
        <div className="text-center">
          <InfoPanel nodes={this.state.graph.nodes.length} />
          <ControlPanel
            onAddClick={this.addNode}
            onLayoutChange={this.onLayoutChange}
            value={this.state.layout}
          />
        </div>
      </div>
    );
  }
}
